@@ -9,7 +9,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2021 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2024 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -26,25 +26,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
-#include "conky.h"
-#include "gui.h"
+
+#include "colours.h"
+
 #include "logging.h"
-#include "x11-color.h"
 
-#ifdef BUILD_X11
-std::unordered_map<Colour, unsigned long, Colour::Hash> Colour::x11_pixels;
-#endif /* BUILD_X11 */
-
-static int hex_nibble_value(char c) {
-  if (c >= '0' && c <= '9') {
-    return c - '0';
-  } else if (c >= 'a' && c <= 'f') {
-    return c - 'a' + 10;
-  } else if (c >= 'A' && c <= 'F') {
-    return c - 'A' + 10;
-  }
-  return -1;
-}
+#include <optional>
 
 Colour Colour::from_argb32(uint32_t argb) {
   Colour out;
@@ -55,29 +42,48 @@ Colour Colour::from_argb32(uint32_t argb) {
   return out;
 }
 
-Colour error_colour{0xff, 0x00, 0x00, 0xff};
+#include "colour-names.cc"
 
-Colour parse_color(const char *name) {
-  unsigned short r, g, b;
-  size_t len = strlen(name);
-  // Parse X11 color names.
-  if (OsLookupColor(-1, name, len, &r, &g, &b)) {
-    Colour out = {(uint8_t)r, (uint8_t)g, (uint8_t)b, 0xff};
-    return out;
+std::optional<Colour> parse_color_name(const std::string &name) {
+  const rgb *value = color_name_hash::in_word_set(name.c_str(), name.length());
+
+  if (value == nullptr) {
+    return std::nullopt;
+  } else {
+    return Colour{value->red, value->green, value->blue};
   }
+}
 
-  // Remove a leading '#' if present.
+std::optional<Colour> parse_hex_color(const std::string &color) {
+  const char *name = color.c_str();
+  size_t len = color.length();
+  // Skip a leading '#' if present.
   if (name[0] == '#') {
     name++;
     len--;
   }
 
-  unsigned char argb[4] = {0xff, 0, 0, 0};
+  static auto hex_nibble_value = [](char c) {
+    if (c >= '0' && c <= '9') {
+      return c - '0';
+    } else if (c >= 'a' && c <= 'f') {
+      return c - 'a' + 10;
+    } else if (c >= 'A' && c <= 'F') {
+      return c - 'A' + 10;
+    }
+    return -1;
+  };
+  const auto none = [&]() {
+    NORM_ERR("can't parse hex color '%s' (%d)", name, len);
+    return std::nullopt;
+  };
+
+  uint8_t argb[4] = {0xff, 0, 0, 0};
   if (len == 3 || len == 4) {
     bool skip_alpha = (len == 3);
-    for (size_t i = 0; i < len; i ++) {
+    for (size_t i = 0; i < len; i++) {
       int nib = hex_nibble_value(name[i]);
-      if (nib < 0) { goto err; }
+      if (nib < 0) { return none(); }
       // Duplicate the nibble, so "#abc" -> 0xaa, 0xbb, 0xcc
       int val = (nib << 4) + nib;
 
@@ -88,27 +94,37 @@ Colour parse_color(const char *name) {
     for (size_t i = 0; i + 1 < len; i += 2) {
       int nib1 = hex_nibble_value(name[i]);
       int nib2 = hex_nibble_value(name[i + 1]);
-      if (nib1 < 0 || nib2 < 0) { goto err; }
+      if (nib1 < 0 || nib2 < 0) { return none(); }
       int val = (nib1 << 4) + nib2;
 
       argb[skip_alpha + i / 2] = val;
     }
   } else {
-    goto err;
+    return none();
   }
 
-  Colour out;
-  out.alpha = argb[0];
-  out.red = argb[1];
-  out.green = argb[2];
-  out.blue = argb[3];
-  return out;
-
-err:
-  NORM_ERR("can't parse X color '%s' (%d)", name, len);
-  return error_colour;
+  return Colour(argb[1], argb[2], argb[3], argb[0]);
 }
 
-Colour parse_color(const std::string &colour) {
-  return parse_color(colour.c_str());
+Colour parse_color(const std::string &color) {
+  std::optional<Colour> result;
+
+#define TRY_PARSER(name)                            \
+  std::optional<Colour> value_##name = name(color); \
+  if (value_##name.has_value()) { return value_##name.value(); }
+
+  TRY_PARSER(parse_color_name)
+  TRY_PARSER(parse_hex_color)
+
+#undef TRY_PARSER
+
+  return ERROR_COLOUR;
+}
+
+Colour::Colour(const std::string &name) {
+  const auto result = parse_color(name);
+  this->red = result.red;
+  this->green = result.green;
+  this->blue = result.blue;
+  this->alpha = result.alpha;
 }
