@@ -9,7 +9,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2021 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2024 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -27,9 +27,12 @@
  *
  */
 
+#include "config.h"
+
 /* local headers */
-#include "core.h"
 #include "algebra.h"
+#include "core.h"
+
 #include "bsdapm.h"
 #include "build.h"
 #include "colour-settings.h"
@@ -40,11 +43,11 @@
 #include "exec.h"
 #include "i8k.h"
 #include "misc.h"
+#include "proc.h"
 #include "text_object.h"
 #ifdef BUILD_IMLIB2
-#include "imlib2.h"
+#include "conky-imlib2.h"
 #endif /* BUILD_IMLIB2 */
-#include "proc.h"
 #ifdef BUILD_MYSQL
 #include "mysql.h"
 #endif /* BUILD_MYSQL */
@@ -836,7 +839,6 @@ struct text_object *construct_text_object(char *s, const char *arg, long line,
   obj->callbacks.free = &gen_free_opaque;
 #endif /* BUILD_GUI */
   END OBJ(conky_version, nullptr) obj_be_plain_text(obj, VERSION);
-  END OBJ(conky_build_date, nullptr) obj_be_plain_text(obj, BUILD_DATE);
   END OBJ(conky_build_arch, nullptr) obj_be_plain_text(obj, BUILD_ARCH);
   END OBJ(downspeed, &update_net_stats)
       parse_net_stat_arg(obj, arg, free_at_crash);
@@ -1212,6 +1214,9 @@ struct text_object *construct_text_object(char *s, const char *arg, long line,
   END OBJ(free_bufcache, &update_meminfo) obj->data.s = STRNDUP_ARG;
   obj->callbacks.print = &print_free_bufcache;
   obj->callbacks.free = &gen_free_opaque;
+  END OBJ(free_cached, &update_meminfo) obj->data.s = STRNDUP_ARG;
+  obj->callbacks.print = &print_free_cached;
+  obj->callbacks.free = &gen_free_opaque;
 #endif /* __linux__ */
 #ifdef __FreeBSD__
   END OBJ(memactive, &update_meminfo) obj->data.s = STRNDUP_ARG;
@@ -1891,9 +1896,10 @@ struct text_object *construct_text_object(char *s, const char *arg, long line,
   obj->callbacks.print = &print_combine;
   obj->callbacks.free = &free_combine;
 #ifdef BUILD_NVIDIA
-  END OBJ_ARG(
-      nvidia, 0,
-      "nvidia needs an argument") if (set_nvidia_query(obj, arg, NONSPECIAL)) {
+  END OBJ_ARG(nvidia, 0, "nvidia needs an argument") if (set_nvidia_query(
+                                                             obj, arg,
+                                                             text_node_t::
+                                                                 NONSPECIAL)) {
     CRIT_ERR_FREE(obj, free_at_crash,
                   "nvidia: invalid argument"
                   " specified: '%s'",
@@ -1903,7 +1909,8 @@ struct text_object *construct_text_object(char *s, const char *arg, long line,
   obj->callbacks.free = &free_nvidia;
   END OBJ_ARG(
       nvidiabar, 0,
-      "nvidiabar needs an argument") if (set_nvidia_query(obj, arg, BAR)) {
+      "nvidiabar needs an argument") if (set_nvidia_query(obj, arg,
+                                                          text_node_t::BAR)) {
     CRIT_ERR_FREE(obj, free_at_crash,
                   "nvidiabar: invalid argument"
                   " specified: '%s'",
@@ -1913,7 +1920,9 @@ struct text_object *construct_text_object(char *s, const char *arg, long line,
   obj->callbacks.free = &free_nvidia;
   END OBJ_ARG(
       nvidiagraph, 0,
-      "nvidiagraph needs an argument") if (set_nvidia_query(obj, arg, GRAPH)) {
+      "nvidiagraph needs an argument") if (set_nvidia_query(obj, arg,
+                                                            text_node_t::
+                                                                GRAPH)) {
     CRIT_ERR_FREE(obj, free_at_crash,
                   "nvidiagraph: invalid argument"
                   " specified: '%s'",
@@ -1923,7 +1932,9 @@ struct text_object *construct_text_object(char *s, const char *arg, long line,
   obj->callbacks.free = &free_nvidia;
   END OBJ_ARG(
       nvidiagauge, 0,
-      "nvidiagauge needs an argument") if (set_nvidia_query(obj, arg, GAUGE)) {
+      "nvidiagauge needs an argument") if (set_nvidia_query(obj, arg,
+                                                            text_node_t::
+                                                                GAUGE)) {
     CRIT_ERR_FREE(obj, free_at_crash,
                   "nvidiagauge: invalid argument"
                   " specified: '%s'",
@@ -2007,6 +2018,13 @@ struct text_object *construct_text_object(char *s, const char *arg, long line,
   END OBJ(pa_card_name, 0) obj->callbacks.print = &print_puau_card_name;
   obj->callbacks.free = &free_pulseaudio;
   init_pulseaudio(obj);
+  END OBJ_IF(if_pa_source_running, 0) obj->callbacks.iftest =
+      &puau_source_running;
+  obj->callbacks.free = &free_pulseaudio;
+  init_pulseaudio(obj);
+  END OBJ_IF(if_pa_source_muted, 0) obj->callbacks.iftest = &puau_source_muted;
+  obj->callbacks.free = &free_pulseaudio;
+  init_pulseaudio(obj);
 #endif /* BUILD_PULSEAUDIO */
 #ifdef BUILD_INTEL_BACKLIGHT
   END OBJ(intel_backlight, 0) obj->callbacks.print = &print_intel_backlight;
