@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2018-2021 François Revol et al.
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2021 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2024 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -24,9 +24,9 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
-#ifdef BUILD_X11
+#include <X11/X.h>
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wvariadic-macros"
 #include <X11/Xutil.h>
@@ -34,22 +34,29 @@
 #include <X11/Xlib.h>
 #endif /* BUILD_XFT */
 #pragma GCC diagnostic pop
-#include "x11.h"
 #ifdef BUILD_XDAMAGE
 #include <X11/extensions/Xdamage.h>
-#endif
+#endif /* BUILD_XDAMAGE */
 #include "fonts.h"
 #ifdef BUILD_IMLIB2
-#include "imlib2.h"
+#include "conky-imlib2.h"
 #endif /* BUILD_IMLIB2 */
-#ifdef BUILD_MOUSE_EVENTS
+#if defined(BUILD_MOUSE_EVENTS) || defined(BUILD_XINPUT)
 #include "mouse-events.h"
-#endif
-#endif /* BUILD_X11 */
+#endif /* BUILD_MOUSE_EVENTS || BUILD_XINPUT */
+#ifdef BUILD_XINPUT
+#include <X11/extensions/XI2.h>
+#include <X11/extensions/XInput2.h>
+#undef COUNT
+#endif /* BUILD_XINPUT */
 
+#include <cstdint>
 #include <iostream>
+#include <map>
 #include <sstream>
+#include <tuple>
 #include <unordered_map>
+#include <vector>
 
 #include "colours.h"
 #include "conky.h"
@@ -57,9 +64,7 @@
 #include "gui.h"
 #include "llua.h"
 
-/* TODO: cleanup global namespace */
-#ifdef BUILD_X11
-
+#include "logging.h"
 #include "x11.h"
 
 // TODO: cleanup externs (move to conky.h ?)
@@ -186,23 +191,15 @@ static void X11_create_window() {
   update_text_area(); /* to get initial size of the window */
 }
 
-#endif /* BUILD_X11 */
-
 namespace conky {
 namespace {
-
-#ifdef BUILD_X11
 conky::display_output_x11 x11_output;
-#else
-conky::disabled_display_output x11_output_disabled("x11", "BUILD_X11");
-#endif
-
 }  // namespace
-extern void init_x11_output() {}
 
-namespace priv {}  // namespace priv
-
-#ifdef BUILD_X11
+template <>
+void register_output<output_t::X11>(display_outputs_t &outputs) {
+  outputs.push_back(&x11_output);
+}
 
 display_output_x11::display_output_x11() : display_output_base("x11") {
   is_graphical = true;
@@ -227,6 +224,9 @@ bool display_output_x11::shutdown() {
   return true;
 }
 
+void process_surface_events(conky::display_output_x11 *surface,
+                            Display *display);
+
 bool display_output_x11::main_loop_wait(double t) {
   /* wait for X event or timeout */
   if (!display || !window.gc) return true;
@@ -312,39 +312,9 @@ bool display_output_x11::main_loop_wait(double t) {
       }
 
       /* update struts */
-      if ((changed != 0) && own_window_type.get(*state) == TYPE_PANEL) {
-        int sidenum = -1;
-
-        DBGP("%s", _(PACKAGE_NAME ": defining struts\n"));
-        fflush(stderr);
-
-        switch (text_alignment.get(*state)) {
-          case TOP_LEFT:
-          case TOP_RIGHT:
-          case TOP_MIDDLE: {
-            sidenum = 2;
-            break;
-          }
-          case BOTTOM_LEFT:
-          case BOTTOM_RIGHT:
-          case BOTTOM_MIDDLE: {
-            sidenum = 3;
-            break;
-          }
-          case MIDDLE_LEFT: {
-            sidenum = 0;
-            break;
-          }
-          case MIDDLE_RIGHT: {
-            sidenum = 1;
-            break;
-          }
-
-          case NONE:
-          case MIDDLE_MIDDLE: /* XXX What about these? */;
-        }
-
-        set_struts(sidenum);
+      if ((changed != 0) && own_window_type.get(*state) == window_type::PANEL) {
+        NORM_ERR("defining struts");
+        set_struts(text_alignment.get(*state));
       }
     }
 #endif
@@ -367,175 +337,7 @@ bool display_output_x11::main_loop_wait(double t) {
     }
   }
 
-  /* handle X events */
-  while (XPending(display) != 0) {
-    XEvent ev;
-    /* indicates whether processed event was consumed */
-    bool consumed = false;
-
-    XNextEvent(display, &ev);
-    switch (ev.type) {
-      case Expose: {
-        XRectangle r;
-        r.x = ev.xexpose.x;
-        r.y = ev.xexpose.y;
-        r.width = ev.xexpose.width;
-        r.height = ev.xexpose.height;
-        XUnionRectWithRegion(&r, x11_stuff.region, x11_stuff.region);
-        XSync(display, False);
-        break;
-      }
-
-      case PropertyNotify: {
-        if (ev.xproperty.state == PropertyNewValue) {
-          get_x11_desktop_info(ev.xproperty.display, ev.xproperty.atom);
-        }
-#ifdef USE_ARGB
-        if (!have_argb_visual) {
-#endif
-          if (ev.xproperty.atom == ATOM(_XROOTPMAP_ID) ||
-              ev.xproperty.atom == ATOM(_XROOTMAP_ID)) {
-            if (forced_redraw.get(*state)) {
-              draw_stuff();
-              next_update_time = get_time();
-              need_to_update = 1;
-            }
-          }
-#ifdef USE_ARGB
-        }
-#endif
-        break;
-      }
-
-#ifdef OWN_WINDOW
-      case ReparentNotify:
-        /* make background transparent */
-        if (own_window.get(*state)) {
-          set_transparent_background(window.window);
-        }
-        break;
-
-      case ConfigureNotify:
-        if (own_window.get(*state)) {
-          /* if window size isn't what expected, set fixed size */
-          if (ev.xconfigure.width != window.width ||
-              ev.xconfigure.height != window.height) {
-            if (window.width != 0 && window.height != 0) { fixed_size = 1; }
-
-            /* clear old stuff before screwing up
-             * size and pos */
-            clear_text(1);
-
-            {
-              XWindowAttributes attrs;
-              if (XGetWindowAttributes(display, window.window, &attrs) != 0) {
-                window.width = attrs.width;
-                window.height = attrs.height;
-              }
-            }
-
-            int border_total = get_border_total();
-
-            text_width = window.width - 2 * border_total;
-            text_height = window.height - 2 * border_total;
-            int mw = this->dpi_scale(maximum_width.get(*state));
-            if (text_width > mw && mw > 0) { text_width = mw; }
-          }
-
-          /* if position isn't what expected, set fixed pos
-           * total_updates avoids setting fixed_pos when window
-           * is set to weird locations when started */
-          /* // this is broken
-          if (total_updates >= 2 && !fixed_pos
-              && (window.x != ev.xconfigure.x
-              || window.y != ev.xconfigure.y)
-              && (ev.xconfigure.x != 0
-              || ev.xconfigure.y != 0)) {
-            fixed_pos = 1;
-          } */
-        }
-        break;
-
-      case ButtonPress:
-#ifdef BUILD_MOUSE_EVENTS
-        if (ev.xbutton.button == 4 || ev.xbutton.button == 5) {
-          consumed = llua_mouse_hook(mouse_scroll_event(&ev.xbutton));
-        } else {
-          consumed = llua_mouse_hook(mouse_press_event(&ev.xbutton));
-        }
-#endif /* BUILD_MOUSE_EVENTS */
-        if (own_window.get(*state)) {
-          /* if an ordinary window with decorations */
-          if ((own_window_type.get(*state) == TYPE_NORMAL &&
-               !TEST_HINT(own_window_hints.get(*state), HINT_UNDECORATED)) ||
-              own_window_type.get(*state) == TYPE_DESKTOP) {
-            /* allow conky to hold input focus. */
-            break;
-          }
-          XUngrabPointer(display, ev.xbutton.time);
-          if (!consumed) {
-            /* forward the click to the desktop window */
-            ev.xbutton.window = window.desktop;
-            ev.xbutton.x = ev.xbutton.x_root;
-            ev.xbutton.y = ev.xbutton.y_root;
-            XSendEvent(display, ev.xbutton.window, False, ButtonPressMask, &ev);
-            XSetInputFocus(display, ev.xbutton.window, RevertToParent,
-                           ev.xbutton.time);
-          }
-        }
-        break;
-
-      case ButtonRelease:
-#ifdef BUILD_MOUSE_EVENTS
-        /* don't report scrollwheel release events */
-        if (ev.xbutton.button != Button4 && ev.xbutton.button != Button5) {
-          llua_mouse_hook(mouse_release_event(&ev.xbutton));
-        }
-#endif /* BUILD_MOUSE_EVENTS */
-        if (own_window.get(*state)) {
-          /* if an ordinary window with decorations */
-          if ((own_window_type.get(*state) == TYPE_NORMAL) &&
-              !TEST_HINT(own_window_hints.get(*state), HINT_UNDECORATED)) {
-            /* allow conky to hold input focus. */
-            break;
-          }
-          /* forward the release to the desktop window */
-          ev.xbutton.window = window.desktop;
-          ev.xbutton.x = ev.xbutton.x_root;
-          ev.xbutton.y = ev.xbutton.y_root;
-          XSendEvent(display, ev.xbutton.window, False, ButtonReleaseMask, &ev);
-        }
-        break;
-#ifdef BUILD_MOUSE_EVENTS
-      /*
-      windows below are notified for the following events as well;
-      can't forward the event without filtering XQueryTree output.
-      */
-      case MotionNotify:
-        llua_mouse_hook(mouse_move_event(&ev.xmotion));
-        break;
-      case EnterNotify:
-        llua_mouse_hook(mouse_enter_event(&ev.xcrossing));
-        break;
-      case LeaveNotify:
-        llua_mouse_hook(mouse_leave_event(&ev.xcrossing));
-        break;
-#endif /* BUILD_MOUSE_EVENTS */
-#endif
-
-      default:
-#ifdef BUILD_XDAMAGE
-        if (ev.type == x11_stuff.event_base + XDamageNotify) {
-          auto *dev = reinterpret_cast<XDamageNotifyEvent *>(&ev);
-
-          XFixesSetRegion(display, x11_stuff.part, &dev->area, 1);
-          XFixesUnionRegion(display, x11_stuff.region2, x11_stuff.region2,
-                            x11_stuff.part);
-        }
-#endif /* BUILD_XDAMAGE */
-        break;
-    }
-  }
+  process_surface_events(this, display);
 
 #ifdef BUILD_XDAMAGE
   if (x11_stuff.damage) {
@@ -581,6 +383,442 @@ bool display_output_x11::main_loop_wait(double t) {
   return true;
 }
 
+enum class x_event_handler {
+  XINPUT_MOTION,
+  MOUSE_INPUT,
+  PROPERTY_NOTIFY,
+  EXPOSE,
+  REPARENT,
+  CONFIGURE,
+  BORDER_CROSSING,
+  DAMAGE,
+};
+
+template <x_event_handler handler>
+bool handle_event(conky::display_output_x11 *surface, Display *display,
+                  XEvent &ev, bool *consumed, void **cookie) {
+  return false;
+}
+
+#ifdef OWN_WINDOW
+template <>
+bool handle_event<x_event_handler::MOUSE_INPUT>(
+    conky::display_output_x11 *surface, Display *display, XEvent &ev,
+    bool *consumed, void **cookie) {
+#ifdef BUILD_XINPUT
+  if (ev.type == ButtonPress || ev.type == ButtonRelease ||
+      ev.type == MotionNotify) {
+    // destroy basic X11 events; and manufacture them later when trying to
+    // propagate XInput ones - this is required because there's no (simple) way
+    // of making sure the lua hook controls both when it only handles XInput
+    // ones.
+    *consumed = true;
+    return true;
+  }
+
+  if (ev.type != GenericEvent || ev.xgeneric.extension != window.xi_opcode)
+    return false;
+
+  if (!XGetEventData(display, &ev.xcookie)) {
+    // already consumed
+    return true;
+  }
+  xi_event_type event_type = ev.xcookie.evtype;
+
+  if (event_type == XI_HierarchyChanged) {
+    auto device_change = reinterpret_cast<XIHierarchyEvent *>(ev.xcookie.data);
+    handle_xi_device_change(device_change);
+    XFreeEventData(display, &ev.xcookie);
+    return true;
+  }
+
+  auto *data = xi_event_data::read_cookie(display, ev.xcookie.data);
+  XFreeEventData(display, &ev.xcookie);
+  if (data == nullptr) {
+    // we ate the cookie, Xi event not handled
+    return true;
+  }
+  *cookie = data;
+
+  Window event_window =
+      query_x11_window_at_pos(display, data->root_x, data->root_y);
+
+  bool same_window = query_x11_top_parent(display, event_window) ==
+                     query_x11_top_parent(display, window.window);
+  bool cursor_over_conky = same_window && data->root_x >= window.x &&
+                           data->root_x < (window.x + window.width) &&
+                           data->root_y >= window.y &&
+                           data->root_y < (window.y + window.height);
+
+  // XInput reports events twice on some hardware (even by 'xinput --test-xi2')
+  auto hash = std::make_tuple(data->serial, data->evtype, data->event);
+  typedef std::map<decltype(hash), Time> MouseEventDebounceMap;
+  static MouseEventDebounceMap debounce{};
+
+  Time now = data->time;
+  bool already_handled = debounce.count(hash) > 0;
+  debounce[hash] = now;
+
+  // clear stale entries
+  for (auto iter = debounce.begin(); iter != debounce.end();) {
+    if (data->time - iter->second > 1000) {
+      iter = debounce.erase(iter);
+    } else {
+      ++iter;
+    }
+  }
+
+  if (already_handled) {
+    *consumed = true;
+    return true;
+  }
+
+#ifdef BUILD_MOUSE_EVENTS
+  // query_result is not window.window in some cases.
+  modifier_state_t mods = x11_modifier_state(data->mods.effective);
+
+  if (data->evtype == XI_Motion) {
+    // TODO: Make valuator_index names configurable?
+
+    bool is_move = data->test_valuator(valuator_t::MOVE_X) ||
+                   data->test_valuator(valuator_t::MOVE_Y);
+    bool is_scroll = data->test_valuator(valuator_t::SCROLL_X) ||
+                     data->test_valuator(valuator_t::SCROLL_Y);
+
+    if (is_move) {
+      static bool cursor_inside = false;
+
+      // generate crossing events
+      if (cursor_over_conky) {
+        if (!cursor_inside) {
+          *consumed = llua_mouse_hook(mouse_crossing_event(
+              mouse_event_t::AREA_ENTER, data->root_x - window.x,
+              data->root_y - window.x, data->root_x, data->root_y));
+        }
+        cursor_inside = true;
+      } else if (cursor_inside) {
+        *consumed = llua_mouse_hook(mouse_crossing_event(
+            mouse_event_t::AREA_LEAVE, data->root_x - window.x,
+            data->root_y - window.x, data->root_x, data->root_y));
+        cursor_inside = false;
+      }
+
+      // generate movement events
+      if (cursor_over_conky) {
+        *consumed = llua_mouse_hook(mouse_move_event(
+            data->event_x, data->event_y, data->root_x, data->root_y, mods));
+      }
+    }
+    if (is_scroll && cursor_over_conky) {
+      scroll_direction_t scroll_direction;
+      auto vertical = data->valuator_relative_value(valuator_t::SCROLL_Y);
+      double vertical_value = vertical.value_or(0.0);
+
+      if (vertical_value != 0.0) {
+        scroll_direction = vertical_value < 0.0 ? scroll_direction_t::UP
+                                                : scroll_direction_t::DOWN;
+      } else {
+        auto horizontal = data->valuator_relative_value(valuator_t::SCROLL_X);
+        double horizontal_value = horizontal.value_or(0.0);
+        if (horizontal_value != 0.0) {
+          scroll_direction = horizontal_value < 0.0 ? scroll_direction_t::LEFT
+                                                    : scroll_direction_t::RIGHT;
+        }
+      }
+
+      if (scroll_direction != scroll_direction_t::UNKNOWN) {
+        *consumed = llua_mouse_hook(
+            mouse_scroll_event(data->event_x, data->event_y, data->root_x,
+                               data->root_y, scroll_direction, mods));
+      }
+    }
+  } else if (cursor_over_conky && (data->evtype == XI_ButtonPress ||
+                                   data->evtype == XI_ButtonRelease)) {
+    if (data->detail >= 4 && data->detail <= 7) {
+      // Handled via motion event valuators, ignoring "backward compatibility"
+      // ones.
+      return true;
+    }
+
+    mouse_event_t type = mouse_event_t::PRESS;
+    if (data->evtype == XI_ButtonRelease) { type = mouse_event_t::RELEASE; }
+
+    mouse_button_t button = x11_mouse_button_code(data->detail);
+    *consumed = llua_mouse_hook(mouse_button_event(type, data->event_x,
+                                                   data->event_y, data->root_x,
+                                                   data->root_y, button, mods));
+  }
+#endif /* BUILD_MOUSE_EVENTS */
+
+#else /* BUILD_XINPUT */
+  if (ev.type != ButtonPress && ev.type != ButtonRelease &&
+      ev.type != MotionNotify)
+    return false;
+  if (ev.xany.window != window.window) return true;  // Skip other windows
+
+#ifdef BUILD_MOUSE_EVENTS
+  switch (ev.type) {
+    case ButtonPress: {
+      modifier_state_t mods = x11_modifier_state(ev.xbutton.state);
+      if (ev.xbutton.button >= 4 &&
+          ev.xbutton.button <= 7) {  // scroll "buttons"
+        scroll_direction_t direction = x11_scroll_direction(ev.xbutton.button);
+        *consumed = llua_mouse_hook(
+            mouse_scroll_event(ev.xbutton.x, ev.xbutton.y, ev.xbutton.x_root,
+                               ev.xbutton.y_root, direction, mods));
+      } else {
+        mouse_button_t button = x11_mouse_button_code(ev.xbutton.button);
+        *consumed = llua_mouse_hook(mouse_button_event(
+            mouse_event_t::PRESS, ev.xbutton.x, ev.xbutton.y, ev.xbutton.x_root,
+            ev.xbutton.y_root, button, mods));
+      }
+      break;
+    }
+    case ButtonRelease: {
+      /* don't report scroll release events */
+      if (ev.xbutton.button >= 4 && ev.xbutton.button <= 7) return true;
+
+      modifier_state_t mods = x11_modifier_state(ev.xbutton.state);
+      mouse_button_t button = x11_mouse_button_code(ev.xbutton.button);
+      *consumed = llua_mouse_hook(mouse_button_event(
+          mouse_event_t::RELEASE, ev.xbutton.x, ev.xbutton.y, ev.xbutton.x_root,
+          ev.xbutton.y_root, button, mods));
+      break;
+    }
+    case MotionNotify: {
+      modifier_state_t mods = x11_modifier_state(ev.xmotion.state);
+      *consumed = llua_mouse_hook(mouse_move_event(ev.xmotion.x, ev.xmotion.y,
+                                                   ev.xmotion.x_root,
+                                                   ev.xmotion.y_root, mods));
+      break;
+    }
+  }
+#endif /* BUILD_MOUSE_EVENTS */
+#endif /* BUILD_XINPUT */
+#ifndef BUILD_MOUSE_EVENTS
+  // always propagate mouse input if not handling mouse events
+  *consumed = false;
+#endif /* BUILD_MOUSE_EVENTS */
+
+  if (!own_window.get(*state)) return true;
+  switch (own_window_type.get(*state)) {
+    case window_type::NORMAL:
+    case window_type::UTILITY:
+      // decorated normal windows always consume events
+      if (!TEST_HINT(own_window_hints.get(*state), window_hints::UNDECORATED)) {
+        *consumed = true;
+      }
+      break;
+    case window_type::DESKTOP:
+      // assume conky is always on bottom; nothing to propagate events to
+      *consumed = true;
+    default:
+      break;
+  }
+
+  return true;
+}
+
+template <>
+bool handle_event<x_event_handler::REPARENT>(conky::display_output_x11 *surface,
+                                             Display *display, XEvent &ev,
+                                             bool *consumed, void **cookie) {
+  if (ev.type != ReparentNotify) return false;
+
+  if (own_window.get(*state)) { set_transparent_background(window.window); }
+  return true;
+}
+
+template <>
+bool handle_event<x_event_handler::CONFIGURE>(
+    conky::display_output_x11 *surface, Display *display, XEvent &ev,
+    bool *consumed, void **cookie) {
+  if (ev.type != ConfigureNotify) return false;
+
+  if (own_window.get(*state)) {
+    /* if window size isn't what expected, set fixed size */
+    if (ev.xconfigure.width != window.width ||
+        ev.xconfigure.height != window.height) {
+      if (window.width != 0 && window.height != 0) { fixed_size = 1; }
+
+      /* clear old stuff before screwing up
+       * size and pos */
+      surface->clear_text(1);
+
+      {
+        XWindowAttributes attrs;
+        if (XGetWindowAttributes(display, window.window, &attrs) != 0) {
+          window.width = attrs.width;
+          window.height = attrs.height;
+        }
+      }
+
+      int border_total = get_border_total();
+
+      text_width = window.width - 2 * border_total;
+      text_height = window.height - 2 * border_total;
+      int mw = surface->dpi_scale(maximum_width.get(*state));
+      if (text_width > mw && mw > 0) { text_width = mw; }
+    }
+
+    /* if position isn't what expected, set fixed pos
+     * total_updates avoids setting fixed_pos when window
+     * is set to weird locations when started */
+    /* // this is broken
+    if (total_updates >= 2 && !fixed_pos
+        && (window.x != ev.xconfigure.x
+        || window.y != ev.xconfigure.y)
+        && (ev.xconfigure.x != 0
+        || ev.xconfigure.y != 0)) {
+      fixed_pos = 1;
+    } */
+  }
+
+  return true;
+}
+
+#ifdef BUILD_MOUSE_EVENTS
+template <>
+bool handle_event<x_event_handler::BORDER_CROSSING>(
+    conky::display_output_x11 *surface, Display *display, XEvent &ev,
+    bool *consumed, void **cookie) {
+  if (ev.type != EnterNotify && ev.type != LeaveNotify) return false;
+  if (window.xi_opcode != 0) return true;  // handled by mouse_input already
+
+  bool not_over_conky = ev.xcrossing.x_root <= window.x ||
+                        ev.xcrossing.y_root <= window.y ||
+                        ev.xcrossing.x_root >= window.x + window.width ||
+                        ev.xcrossing.y_root >= window.y + window.height;
+
+  if ((not_over_conky && ev.xcrossing.type == LeaveNotify) ||
+      (!not_over_conky && ev.xcrossing.type == EnterNotify)) {
+    llua_mouse_hook(mouse_crossing_event(
+        ev.xcrossing.type == EnterNotify ? mouse_event_t::AREA_ENTER
+                                         : mouse_event_t::AREA_LEAVE,
+        ev.xcrossing.x, ev.xcrossing.y, ev.xcrossing.x_root,
+        ev.xcrossing.y_root));
+  }
+  return true;
+}
+#endif /* BUILD_MOUSE_EVENTS */
+#endif /* OWN_WINDOW */
+
+template <>
+bool handle_event<x_event_handler::PROPERTY_NOTIFY>(
+    conky::display_output_x11 *surface, Display *display, XEvent &ev,
+    bool *consumed, void **cookie) {
+  if (ev.type != PropertyNotify) return false;
+
+  if (ev.xproperty.state == PropertyNewValue) {
+    get_x11_desktop_info(ev.xproperty.display, ev.xproperty.atom);
+  }
+
+#ifdef USE_ARGB
+  if (have_argb_visual) return true;
+#endif
+
+  if (ev.xproperty.atom == ATOM(_XROOTPMAP_ID) ||
+      ev.xproperty.atom == ATOM(_XROOTMAP_ID)) {
+    if (forced_redraw.get(*state)) {
+      draw_stuff();
+      next_update_time = get_time();
+      need_to_update = 1;
+    }
+  }
+  return true;
+}
+
+template <>
+bool handle_event<x_event_handler::EXPOSE>(conky::display_output_x11 *surface,
+                                           Display *display, XEvent &ev,
+                                           bool *consumed, void **cookie) {
+  if (ev.type != Expose) return false;
+
+  XRectangle r;
+  r.x = ev.xexpose.x;
+  r.y = ev.xexpose.y;
+  r.width = ev.xexpose.width;
+  r.height = ev.xexpose.height;
+  XUnionRectWithRegion(&r, x11_stuff.region, x11_stuff.region);
+  XSync(display, False);
+  return true;
+}
+
+#ifdef BUILD_XDAMAGE
+template <>
+bool handle_event<x_event_handler::DAMAGE>(conky::display_output_x11 *surface,
+                                           Display *display, XEvent &ev,
+                                           bool *consumed, void **cookie) {
+  if (ev.type != x11_stuff.event_base + XDamageNotify) return false;
+
+  auto *dev = reinterpret_cast<XDamageNotifyEvent *>(&ev);
+
+  XFixesSetRegion(display, x11_stuff.part, &dev->area, 1);
+  XFixesUnionRegion(display, x11_stuff.region2, x11_stuff.region2,
+                    x11_stuff.part);
+  return true;
+}
+#endif /* BUILD_XDAMAGE */
+
+/// Handles all events conky can receive.
+///
+/// @return true if event should move input focus to conky
+bool process_event(conky::display_output_x11 *surface, Display *display,
+                   XEvent ev, bool *consumed, void **cookie) {
+#define HANDLE_EV(event)                                                   \
+  if (handle_event<x_event_handler::event>(surface, display, ev, consumed, \
+                                           cookie)) {                      \
+    return true;                                                           \
+  }
+
+  HANDLE_EV(XINPUT_MOTION)
+  HANDLE_EV(MOUSE_INPUT)
+  HANDLE_EV(PROPERTY_NOTIFY)
+
+  // only accept remaining events if they're sent to Conky.
+  if (ev.xany.window != window.window) return false;
+
+  HANDLE_EV(EXPOSE)
+  HANDLE_EV(REPARENT)
+  HANDLE_EV(CONFIGURE)
+  HANDLE_EV(BORDER_CROSSING)
+  HANDLE_EV(DAMAGE)
+
+#undef HANDLE_EV
+
+  // event not handled
+  return false;
+}
+
+void process_surface_events(conky::display_output_x11 *surface,
+                            Display *display) {
+  int pending = XPending(display);
+  if (pending == 0) return;
+
+  DBGP2("Processing %d X11 events...", pending);
+
+  /* handle X events */
+  while (XPending(display) != 0) {
+    XEvent ev;
+    XNextEvent(display, &ev);
+
+    /*
+    indicates whether processed event was consumed; true by default so we
+    don't propagate handled events unless they explicitly state they haven't
+    been consumed.
+    */
+    bool consumed = true;
+    void *cookie = nullptr;
+    bool handled = process_event(surface, display, ev, &consumed, &cookie);
+
+    if (!consumed) { propagate_x11_event(ev, cookie); }
+
+    if (cookie != nullptr) { free(cookie); }
+  }
+
+  DBGP2("Done processing %d events.", pending);
+}
+
 void display_output_x11::sigterm_cleanup() {
   XDestroyRegion(x11_stuff.region);
   x11_stuff.region = nullptr;
@@ -607,9 +845,6 @@ void display_output_x11::cleanup() {
     XDestroyRegion(x11_stuff.region);
     x11_stuff.region = nullptr;
   }
-#ifdef BUILD_XFT
-  FcFini();
-#endif /* BUILD_XFT */
 }
 
 void display_output_x11::set_foreground_color(Colour c) {
@@ -620,11 +855,11 @@ void display_output_x11::set_foreground_color(Colour c) {
   }
 #endif /* BUILD_ARGB */
   XSetForeground(display, window.gc,
-                 current_color.to_x11_color(display, screen));
+                 current_color.to_x11_color(display, screen, have_argb_visual));
 }
 
 int display_output_x11::calc_text_width(const char *s) {
-  size_t slen = strlen(s);
+  std::size_t slen = strlen(s);
 #ifdef BUILD_XFT
   if (use_xft.get(*state)) {
     XGlyphInfo gi;
@@ -649,7 +884,7 @@ void display_output_x11::draw_string_at(int x, int y, const char *s, int w) {
     XColor c{};
     XftColor c2{};
 
-    c.pixel = current_color.to_x11_color(display, screen);
+    c.pixel = current_color.to_x11_color(display, screen, have_argb_visual);
     // query color on custom colormap
     XQueryColor(display, window.colourmap, &c);
 
@@ -703,15 +938,19 @@ void display_output_x11::draw_arc(int x, int y, int w, int h, int a1, int a2) {
 }
 
 void display_output_x11::move_win(int x, int y) {
+#ifdef OWN_WINDOW
   window.x = x;
   window.y = y;
   XMoveWindow(display, window.window, x, y);
+#endif /* OWN_WINDOW */
 }
 
-int display_output_x11::dpi_scale(int value) {
+const size_t PIXELS_PER_INCH = 96;
+template <typename T, typename>
+T display_output_x11::dpi_scale(T value) {
 #if defined(BUILD_XFT)
   if (use_xft.get(*state) && xft_dpi > 0) {
-    return (value * xft_dpi + (value > 0 ? 48 : -48)) / 96;
+    return (value * xft_dpi + (value > 0 ? 48 : -48)) / PIXELS_PER_INCH;
   } else {
     return value;
   }
@@ -902,6 +1141,4 @@ void display_output_x11::load_fonts(bool utf8) {
   }
 }
 
-#endif /* BUILD_X11 */
-
 }  // namespace conky
