@@ -8,7 +8,7 @@
  *
  * Please see COPYING for details
  *
- * Copyright (c) 2005-2021 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2024 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -57,8 +57,7 @@
 #endif /* BUILD_BUILTIN_CONFIG */
 
 static void print_version() {
-  std::cout << _(PACKAGE_NAME " " VERSION " compiled " BUILD_DATE
-                              " for " BUILD_ARCH
+  std::cout << _(PACKAGE_NAME " " VERSION " compiled for " BUILD_ARCH
                               "\n"
                               "\nCompiled in features:\n\n"
                               "System config file: " SYSTEM_CONFIG_FILE
@@ -172,6 +171,9 @@ static void print_version() {
 #ifdef BUILD_XFT
             << _("  * Xft\n")
 #endif /* BUILD_XFT */
+#ifdef BUILD_XINPUT
+            << _("  * Xinput\n")
+#endif /* BUILD_XINPUT */
 #ifdef BUILD_ARGB
             << _("  * ARGB visual\n")
 #endif /* BUILD_ARGB */
@@ -179,9 +181,18 @@ static void print_version() {
             << _("  * Own window\n")
 #endif
 #ifdef BUILD_MOUSE_EVENTS
-            << _("  * Mouse evenets\n")
-#endif
+            << _("  * Mouse events\n")
+#endif /* BUILD_MOUSE_EVENTS */
 #endif /* BUILD_X11 */
+#ifdef BUILD_WAYLAND
+            << _(" Wayland:\n")
+#ifdef BUILD_ARGB
+            << _("  * ARGB visual\n")
+#endif /* BUILD_ARGB */
+#ifdef BUILD_MOUSE_EVENTS
+            << _("  * Mouse events\n")
+#endif /* BUILD_MOUSE_EVENTS */
+#endif /* BUILD_WAYLAND */
 #if defined BUILD_AUDACIOUS || defined BUILD_CMUS || defined BUILD_MPD || \
     defined BUILD_MOC || defined BUILD_XMMS2
             << _("\n Music detection:\n")
